'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var axios = require('axios');
var https = require('https');
var dateFns = require('date-fns');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);
var https__default = /*#__PURE__*/_interopDefaultLegacy(https);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

class JumboObject {
    constructor(jumbo, authRequired) {
        this.jumbo = jumbo;
        this.authRequired = authRequired;
    }
}

class Product extends JumboObject {
    /**
     * Gets product from ID
     * @param productId Product ID
     */
    getProductFromId(productId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`products/${productId}`, additionalRequestOptions);
        });
    }
    /**
     * Get products from given product name
     * @param productName Product name to search for
     * @param options Options for the query
     * @param options.offset Offset in search (default 0)
     * @param options.limit Amount of products returned (default 10)
     * @param options.filter Filters (from ProductFilter)
     * @param options.sort Sort options (from ProductSortOptions)
     */
    getProductsFromName(productName, options, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const totalQuery = {
                q: productName,
                offset: ((options === null || options === void 0 ? void 0 : options.offset) || 0).toString(),
                limit: ((options === null || options === void 0 ? void 0 : options.limit) || 10).toString(),
                sort: ((options === null || options === void 0 ? void 0 : options.sort) || '').toString()
            };
            if (options === null || options === void 0 ? void 0 : options.filters) {
                totalQuery['filters'] = this.translateProductFilterToQuery(options.filters);
            }
            // First get query results as productQueryModel
            const products = yield this.jumbo.get(`search`, Object.assign({ query: Object.assign({}, totalQuery) }, additionalRequestOptions));
            // Then create a ProductModel for every ProductQuery product
            const result = [];
            for (var key in products.products.data) {
                const product = {
                    product: {
                        data: products.products.data[key]
                    }
                };
                result.push(product);
            }
            // Return array of ProductModels
            return result;
        });
    }
    /**
     * Translates the product filters to a usable query
     */
    translateProductFilterToQuery(filter) {
        // Simply join all of the numbers together with spaces
        const out = [];
        if (filter.diet) {
            out.push(filter.diet.map((diet) => diet.toString()).join(' '));
        }
        if (filter.allergens) {
            out.push(filter.allergens.map((allergy) => allergy.toString()).join(' '));
        }
        if (filter.other) {
            out.push(filter.other.map((other) => other.toString()).join(' '));
        }
        return out.join(' ');
    }
}
/**
 * The different sort options for products
 */
exports.ProductSortOptions = void 0;
(function (ProductSortOptions) {
    ProductSortOptions["POPULAR"] = "P_Sales|1";
    ProductSortOptions["PRICE_ASC"] = "P_Price|0";
    ProductSortOptions["PRICE_DESC"] = "P_Price|1";
    ProductSortOptions["ALPHAB_ASC"] = "P_Title|0";
    ProductSortOptions["ALPHAB_DESC"] = "P_Title|1";
    ProductSortOptions["PER_WEIGHT_MEASURE"] = "pricePerWeightMeasure|0";
})(exports.ProductSortOptions || (exports.ProductSortOptions = {}));
exports.ProductDietFilter = void 0;
(function (ProductDietFilter) {
    ProductDietFilter[ProductDietFilter["Bio"] = 4294960557] = "Bio";
    ProductDietFilter[ProductDietFilter["LactoseIntolerant"] = 4294960541] = "LactoseIntolerant";
    ProductDietFilter[ProductDietFilter["Vegan"] = 204294960595] = "Vegan";
    ProductDietFilter[ProductDietFilter["GlutenIntolerant"] = 4294960639] = "GlutenIntolerant";
    ProductDietFilter[ProductDietFilter["NoAddedSugar"] = 4294960544] = "NoAddedSugar";
    ProductDietFilter[ProductDietFilter["NoAddedSalt"] = 4294960120] = "NoAddedSalt";
    ProductDietFilter[ProductDietFilter["Vegetarian"] = 4294960623] = "Vegetarian";
    ProductDietFilter[ProductDietFilter["LowFat"] = 4294960596] = "LowFat";
    ProductDietFilter[ProductDietFilter["LowSugar"] = 4294952996] = "LowSugar";
    ProductDietFilter[ProductDietFilter["NoSugar"] = 4294960685] = "NoSugar";
    ProductDietFilter[ProductDietFilter["NoFat"] = 4294960453] = "NoFat";
    ProductDietFilter[ProductDietFilter["FatPoor"] = 4294926481] = "FatPoor";
    ProductDietFilter[ProductDietFilter["Halal"] = 4294960615] = "Halal";
    ProductDietFilter[ProductDietFilter["Kosher"] = 4294960433] = "Kosher";
})(exports.ProductDietFilter || (exports.ProductDietFilter = {}));
exports.ProductAllergenFilter = void 0;
(function (ProductAllergenFilter) {
    ProductAllergenFilter[ProductAllergenFilter["Lactose"] = 4294960577] = "Lactose";
    ProductAllergenFilter[ProductAllergenFilter["Gluten"] = 4294960640] = "Gluten";
    ProductAllergenFilter[ProductAllergenFilter["Diary"] = 4294960597] = "Diary";
    ProductAllergenFilter[ProductAllergenFilter["Milk"] = 4294960576] = "Milk";
    ProductAllergenFilter[ProductAllergenFilter["Soy"] = 4294960064] = "Soy";
    ProductAllergenFilter[ProductAllergenFilter["Peanuts"] = 4294960574] = "Peanuts";
    ProductAllergenFilter[ProductAllergenFilter["Nuts"] = 4294960575] = "Nuts";
    ProductAllergenFilter[ProductAllergenFilter["Wheat"] = 4294960573] = "Wheat";
    ProductAllergenFilter[ProductAllergenFilter["Eggs"] = 4294960578] = "Eggs";
})(exports.ProductAllergenFilter || (exports.ProductAllergenFilter = {}));

class Store extends JumboObject {
    /**
     * Gets store from store ID
     * @param storeId Store ID
     */
    getStoreFromId(storeId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.jumbo.get(`stores/${storeId}`, additionalRequestOptions);
        });
    }
    /**
     * Find stores based on given longitude, latitude and distance (optional)
     * @param options Options for the search
     * @param options.long Longitude
     * @param options.lat Latitude
     * @param options.distance (Optional) Maximum search radius (default none)
     * @param options.limit Number of stores to retrieve (default 20)
     * @param options.offset Offset in search (default 0)
     */
    getStoresFromLongLat(options, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const stores = yield this.jumbo.get(`stores`, Object.assign({ query: {
                    longitude: options.long.toString(),
                    latitude: options.lat.toString(),
                    distance: (options.distance || '').toString(),
                    count: (options.limit || 20).toString(),
                    offset: (options.offset || 0).toString()
                } }, additionalRequestOptions));
            const result = [];
            for (var key in stores.stores.data) {
                const store = {
                    store: {
                        data: stores.stores.data[key]
                    }
                };
                result.push(store);
            }
            return result;
        });
    }
}

class TokenHandler {
    constructor(jumbo, username, password, token) {
        this.jumbo = jumbo;
        this.username = username;
        this.password = password;
        this.token = token;
        if (token) {
            this.jumboToken = token;
            this.Ready = Promise.resolve();
            return;
        }
        this.Ready = new Promise((resolve, reject) => {
            this.generateToken()
                .then(() => {
                resolve(undefined);
            })
                .catch(reject);
        });
    }
    /**
     * Function that can be called to generate token (if there is no token yet)
     */
    generateToken() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.jumboToken) {
                this.jumboToken = yield this.login();
            }
        });
    }
    /**
     * Function that is used to login to Jumbo and get a token
     */
    login() {
        return __awaiter(this, void 0, void 0, function* () {
            // To get a token, we simply initialize a POST request to /users/login with the info
            let body = {
                password: this.password,
                username: this.username
            };
            // Send POST request with info
            let response = yield this.jumbo.post(`users/login`, JSON.stringify(body), undefined, false, true);
            // Save token
            return yield response.headers['x-jumbo-token'];
        });
    }
    /**
     * Simple function that returns the token (assuming it exists)
     */
    getToken() {
        // TODO: refresh token if needed
        return this.jumboToken;
    }
}

class Order extends JumboObject {
    /**
     * Gets order from ID
     * @param orderId Order ID
     */
    getMyOrderById(orderId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get order from endpoint
            const order = yield this.jumbo.get(`users/me/orders/${orderId}`, additionalRequestOptions, this.authRequired);
            // Convert times to dates
            return this.convertOrderResponseToOrderModel(order.order.data);
        });
    }
    /**
     * Returns all the users' orders
     * @param options Options for search
     * @param options.offset Offset in search (default 0)
     * @param options.limit Amount of lists to retrieve (default 10)
     */
    getMyOrders(options, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            // Query all orders as orderQueryModel
            const orders = yield this.jumbo.get(`users/me/orders`, Object.assign({ query: {
                    offset: ((options === null || options === void 0 ? void 0 : options.offset) || 0).toString(),
                    count: ((options === null || options === void 0 ? void 0 : options.limit) || 10).toString()
                } }, additionalRequestOptions), this.authRequired);
            // Convert responses to models
            return orders.orders.data.map((orderResponse) => {
                return this.convertOrderResponseToOrderModel(orderResponse);
            });
        });
    }
    /**
     * Shortcut function to return latest (first) order
     */
    getMyLatestOrder(additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders({ limit: 1 }, additionalRequestOptions);
            return orders[0];
        });
    }
    /**
     * Returns all of the user's orders with a certain status
     * @param status Status of the order (Processing, Open, Completed)
     */
    getMyOrdersByStatus(status, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders(undefined, additionalRequestOptions);
            return orders.filter((order) => {
                return order.order.data.status === status;
            });
        });
    }
    /**
     * Gets the user's relevant orders (which includes shipping time)
     */
    getMyRelevantOrders(additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders(undefined, Object.assign({ query: {
                    relevant: 'true'
                } }, additionalRequestOptions));
            return orders;
        });
    }
    /**
     * Converts response to actual data by first converting unix timestamps to dates
     * @param order OrderResponse to convert
     */
    convertOrderResponseToOrderModel(order) {
        // Simply convert the delivery object and cut-off date
        const orderModel = {
            order: {
                data: Object.assign(Object.assign({}, order), { delivery: this.convertDeliveryTimesToDates(order.delivery), orderCutOffDate: this.convertUnixDateToDate(order.orderCutOffDate), shipping: undefined })
            }
        };
        // Add shipping if required
        if (order.shipping) {
            orderModel.order.data.shipping = this.convertShippingTimesToDates(order.shipping);
        }
        return orderModel;
    }
    /**
     * Converts shipping unix timestamps of order to actual dates
     * @param shipping order.data.shipping
     */
    convertShippingTimesToDates(shipping) {
        const shippingDate = {
            plannedETAStart: this.convertUnixDateToDate(shipping.plannedETAStart),
            plannedETAEnd: this.convertUnixDateToDate(shipping.plannedETAEnd),
            unknownLiveETA: shipping.unknownLiveETA
        };
        if (shipping.liveETA) {
            shippingDate.liveETA = this.convertUnixDateToDate(shipping.liveETA);
        }
        return shippingDate;
    }
    /**
     * Converts delivery unix timestamps of order to actual dates
     * @param delivery order.data.delivery
     */
    convertDeliveryTimesToDates(delivery) {
        // First convert these 3 separately
        const date = this.convertUnixDateToDate(delivery.date);
        const startDate = this.convertUnixDateToDate(delivery.startDateTime);
        const endDate = this.convertUnixDateToDate(delivery.endDateTime);
        // Add them to return object
        const deliveryDate = {
            address: delivery.address,
            date: date,
            endDateTime: endDate,
            location: delivery.location,
            price: delivery.price,
            startDateTime: startDate,
            time: delivery.time
        };
        // Only convert and add collection date if it's in the original delivery data
        if (delivery.collectionDateTime) {
            const collectionDate = this.convertUnixDateToDate(delivery.collectionDateTime);
            deliveryDate.collectionDateTime = collectionDate;
        }
        return deliveryDate;
    }
    /**
     * Converts given unix timestamp to a Date
     */
    convertUnixDateToDate(unixDate) {
        return dateFns.fromUnixTime(this.formatUnixTimestamp(unixDate));
    }
    /**
     * Strips the last 3 digits off the timestamp to properly convert
     */
    formatUnixTimestamp(unixDate) {
        return (unixDate - (unixDate % 1000)) / 1000;
    }
}
exports.OrderStatus = void 0;
(function (OrderStatus) {
    OrderStatus["Processing"] = "PROCESSING";
    OrderStatus["Open"] = "OPEN";
    OrderStatus["ReadyToDeliver"] = "READY_TO_DELIVER";
    OrderStatus["Completed"] = "PICKED_UP";
})(exports.OrderStatus || (exports.OrderStatus = {}));

class Promotion extends JumboObject {
    /**
     * Get all promotions from a given store (by ID)
     * @param storeId Store ID
     */
    getPromotionsFromStore(storeId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`promotion-overview`, Object.assign({ query: {
                    store_id: storeId.toString()
                } }, additionalRequestOptions));
        });
    }
}

class Recipe extends JumboObject {
    /**
     * Get recipe from ID
     * @param recipeId Recipe ID
     */
    getRecipeFromId(recipeId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`recipes/${recipeId}`, additionalRequestOptions);
        });
    }
    /**
     * Get recipes from given recipe name
     * @param recipeName Recipe name to search for
     * @param options Options for the search
     * @param options.offset Offset in search (default 0)
     * @param options.limit Amount of products returned (default 10)
     */
    getRecipesFromName(recipeName, options, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const recipes = yield this.jumbo.get(`recipes`, Object.assign({ query: {
                    q: recipeName,
                    offset: ((options === null || options === void 0 ? void 0 : options.offset) || 0).toString(),
                    limit: ((options === null || options === void 0 ? void 0 : options.limit) || 10).toString()
                } }, additionalRequestOptions));
            const result = [];
            for (var key in recipes.recipes.data) {
                const recipe = {
                    recipe: {
                        data: recipes.recipes.data[key]
                    }
                };
                result.push(recipe);
            }
            return result;
        });
    }
    /**
     * Get recipes from given filter ID
     * @param filterId Recipe filter ID
     * @param options Options for the search
     * @param options.offset Offset in search (default 0)
     * @param options.limit Amount of products returned (default 10)
     */
    getRecipesFromFilterId(filterId, options, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const recipes = yield this.jumbo.get(`recipes`, Object.assign({ query: {
                    filterId: filterId.toString(),
                    offset: ((options === null || options === void 0 ? void 0 : options.offset) || 0).toString(),
                    limit: ((options === null || options === void 0 ? void 0 : options.limit) || 10).toString()
                } }, additionalRequestOptions));
            const result = [];
            for (var key in recipes.recipes.data) {
                const recipe = {
                    recipe: {
                        data: recipes.recipes.data[key]
                    }
                };
                result.push(recipe);
            }
            return result;
        });
    }
}

class User extends JumboObject {
    /**
     * Returns info of logged in user
     */
    getMyInfo(additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`users/me`, additionalRequestOptions, this.authRequired);
        });
    }
}

class Category extends JumboObject {
    /**
     * Get category from category ID
     * @param categoryId Category ID
     */
    getCategoryFromId(categoryId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.jumbo.get(`categories`, Object.assign({ query: {
                    id: categoryId.toString()
                } }, additionalRequestOptions));
        });
    }
}

class List extends JumboObject {
    /**
     * Returns all of your lists (login required)
     */
    getMyLists(additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/mylists`, additionalRequestOptions, true);
        });
    }
    /**
     * Returns a single list given a list ID
     * @param listId ID of list
     */
    getListFromId(listId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/${listId}`, additionalRequestOptions, false);
        });
    }
    /**
     * Returns a list of lists that match the given name (name is case sensitive and very specific)
     * @param listName Name of list (category)
     * @param options Options for search
     * @param options.offset Offset in search (default 0)
     * @param options.limit Amount of lists to retrieve (default 10)
     */
    getListsByName(listName, options, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/search`, Object.assign({ query: {
                    offset: ((options === null || options === void 0 ? void 0 : options.offset) || 0).toString(),
                    limit: ((options === null || options === void 0 ? void 0 : options.limit) || 10).toString(),
                    q: listName
                } }, additionalRequestOptions), false);
        });
    }
    /**
     * Shortcut function to get the most popular lists
     * @param options Options for search
     */
    getPopularLists(options, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getListsByName('', options, additionalRequestOptions);
        });
    }
    /**
     * Gets all items from a given list
     * @param listId ID of list
     * @param options Options for search
     * @param options.offset Offset in search (default 0)
     * @param options.limit Amount of lists to retrieve (default 10)
     */
    getItemsFromList(listId, options, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/${listId}/items`, Object.assign({ query: {
                    offset: ((options === null || options === void 0 ? void 0 : options.offset) || 0).toString(),
                    limit: ((options === null || options === void 0 ? void 0 : options.limit) || 10).toString()
                } }, additionalRequestOptions), false);
        });
    }
    /**
     * Gets all smart lists (i.e. last purchased) of user (login required)
     */
    getMySmartLists(additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`users/me/smart-lists`, additionalRequestOptions, true);
        });
    }
    /**
     * Gets all the lists the user currently follows (login required)
     */
    getMyFollowedLists(additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/following`, additionalRequestOptions, true);
        });
    }
    /**
     * Checks whether the user is following the given list (login required)
     * @param listId ID of list
     */
    isFollowingList(listId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const isFollowing = yield this.jumbo.get(`lists/${listId}/following`, additionalRequestOptions, true);
            return isFollowing.isFollowing;
        });
    }
    /**
     * Makes the user follow the given list (login required)
     * @param listId ID of list to follow
     */
    followList(listId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.put(`lists/${listId}/follow`, undefined, additionalRequestOptions, true);
        });
    }
    /**
     * Makes the user unfollow the given list (login required)
     * @param listId ID of list to unfollow
     */
    unfollowList(listId, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.put(`lists/${listId}/unfollow`, undefined, additionalRequestOptions, true);
        });
    }
}

class Basket extends JumboObject {
    /**
     * Gets basket
     */
    getMyBasket(additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`basket`, additionalRequestOptions);
        });
    }
    /**
     * Updates basket with items, overwriting existing items
     * @param productJson Items with quantity, sku and unit
     */
    updateBasket(productJson, additionalRequestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.put(`basket`, productJson, additionalRequestOptions);
        });
    }
}

class Jumbo {
    /**
     * @param options Options for the client
     * @param options.username Jumbo account username (e-mail)
     * @param options.password Jumbo account password
     * @param options.token Jumbo access token
     * @param options.verbose Whether to log requests (default false)
     * @param options.axiosConfig Axios configuration (defaults to TLSv1.2)
     * @param options.apiVersion Jumbo API version (defaults to 17)
     */
    constructor(options) {
        this.endpoint = 'https://mobileapi.jumbo.com/';
        // Create https agent for TLSv1.2 or less (API doesn't respond to TLSv1.3+)
        this.client = (options === null || options === void 0 ? void 0 : options.axiosConfig)
            ? axios__default['default'].create(options.axiosConfig)
            : axios__default['default'].create({
                httpsAgent: new https__default['default'].Agent({
                    maxVersion: 'TLSv1.2'
                })
            });
        this.endpoint = (options === null || options === void 0 ? void 0 : options.apiVersion) ? this.endpoint + `v${options.apiVersion}/` : this.endpoint + 'v17/';
        // Set separate classes
        this.jumboBasket = new Basket(this, false);
        this.jumboCategory = new Category(this, false);
        this.jumboList = new List(this, false);
        this.jumboOrder = new Order(this, true);
        this.jumboProduct = new Product(this, false);
        this.jumboPromotion = new Promotion(this, false);
        this.jumboRecipe = new Recipe(this, false);
        this.jumboStore = new Store(this, false);
        this.jumboUser = new User(this, true);
        // Login using given username and password
        if ((options === null || options === void 0 ? void 0 : options.username) && (options === null || options === void 0 ? void 0 : options.password)) {
            this.login(options.username, options.password);
        }
        // Login using given token
        if (options === null || options === void 0 ? void 0 : options.token) {
            this.loginWithToken(options.token);
        }
        this.verbose = (options === null || options === void 0 ? void 0 : options.verbose) || false;
    }
    basket() {
        return this.jumboBasket;
    }
    category() {
        return this.jumboCategory;
    }
    list() {
        return this.jumboList;
    }
    order() {
        return this.jumboOrder;
    }
    product() {
        return this.jumboProduct;
    }
    promotion() {
        return this.jumboPromotion;
    }
    recipe() {
        return this.jumboRecipe;
    }
    store() {
        return this.jumboStore;
    }
    user() {
        return this.jumboUser;
    }
    /**
     * Function that creates a new TokenHandler for given username and password
     * @param username Jumbo account username (e-mail)
     * @param password Jumbo account password
     */
    login(username, password) {
        this.tokenHandler = new TokenHandler(this, username, password);
    }
    /**
     * Function that creates a new TokenHandler for given access token
     * @param token Jumbo access token
     */
    loginWithToken(token) {
        this.tokenHandler = new TokenHandler(this, '', '', token);
    }
    /**
     * PUT request
     * @param path Endpoint URL (without start)
     * @param body Body of PUT (if any)
     * @param additionalRequestOptions Any additional headers or queries
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    put(path, body, additionalRequestOptions, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, exports.requestMethod.PUT, body, additionalRequestOptions, authRequired, fullResponse);
        });
    }
    /**
     * POST request
     * @param path Endpoint URL (without start)
     * @param body Body of POST
     * @param additionalRequestOptions Any additional headers or queries
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    post(path, body, additionalRequestOptions, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, exports.requestMethod.POST, body, additionalRequestOptions, authRequired, fullResponse);
        });
    }
    /**
     * GET request
     * @param path Endpoint URL (without start)
     * @param additionalRequestOptions Any additional headers or queries
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    get(path, additionalRequestOptions, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, exports.requestMethod.GET, undefined, additionalRequestOptions, authRequired, fullResponse);
        });
    }
    /**
     * Generic request method
     * @param path Endpoint URL (without start)
     * @param method Request method (GET, POST, PUT, DELETE)
     * @param body Body in case of POST and PUT
     * @param additionalRequestOptions Any additional headers or queries
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    request(path, method, body, additionalRequestOptions, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            // If auth is required and we don't have a token yet, we should create one
            if (authRequired) {
                if (!this.tokenHandler) {
                    throw new Error(`You must be logged in to access this path: ${this.endpoint + path}`);
                }
                else {
                    // If the tokenHandler doesn't have a token yet, make sure it gets one
                    yield this.tokenHandler.Ready;
                }
            }
            // Create initial header properties
            let requestHeaders = this.createHeader(authRequired, additionalRequestOptions === null || additionalRequestOptions === void 0 ? void 0 : additionalRequestOptions.headers);
            // Add query to URL if given
            let url = this.createURL(path, additionalRequestOptions === null || additionalRequestOptions === void 0 ? void 0 : additionalRequestOptions.query);
            // Log if verbose
            if (this.verbose) {
                console.log(url);
                console.log(method);
                console.log(requestHeaders);
                void (body && console.log(body));
            }
            // Make request
            let response = yield this.client.request({
                method: method,
                url: url,
                headers: requestHeaders,
                data: body
            });
            // Throw error if response not ok
            if (!response.statusText) {
                const text = response.data;
                throw new Error(`${response.statusText}: ${text}`);
            }
            if (fullResponse) {
                return response;
            }
            // Return response in JSON format
            return response.data;
        });
    }
    /**
     * Helper function to create headers for request
     * @param extraHeaders Any extra header options
     */
    createHeader(authRequired, extraHeaders) {
        // Create header
        let headers = Object.assign({ 'Content-Type': 'application/json', 'User-Agent': 'jumbo-wrapper' }, extraHeaders);
        if (authRequired && this.tokenHandler) {
            headers['x-jumbo-token'] = this.tokenHandler.getToken();
        }
        else if (authRequired && !this.tokenHandler) {
            throw new Error('You must be logged in to use this function');
        }
        // Return the headers
        return headers;
    }
    /**
     * Helper function to create request URL
     * @param path Path to endpoint (without ENDPOINT in .env)
     * @param query Any query options
     */
    createURL(path, query) {
        let url;
        // Add query if given
        if (query) {
            const params = new URLSearchParams(query);
            url = this.endpoint + path + '?' + params;
        }
        else {
            url = this.endpoint + path;
        }
        // Return URL
        return url;
    }
}
/**
 * Simple enum for different request methods
 */
exports.requestMethod = void 0;
(function (requestMethod) {
    requestMethod["GET"] = "GET";
    requestMethod["POST"] = "POST";
    requestMethod["PUT"] = "PUT";
})(exports.requestMethod || (exports.requestMethod = {}));

exports.Basket = Basket;
exports.Category = Category;
exports.Jumbo = Jumbo;
exports.List = List;
exports.Order = Order;
exports.Product = Product;
exports.Promotion = Promotion;
exports.Recipe = Recipe;
exports.Store = Store;
exports.User = User;
