import { JumboObject, PaginationOptions } from '../base/jumboObject';
import { AdditionalRequestOptions } from '../jumbo';
import { ListItemsModel } from './listItemsModel';
import { ListModel, ListQueryModel } from './listModel';
import { PublicListsModel } from './publicListsModel';
export interface ListOptions extends PaginationOptions {
}
export declare class List extends JumboObject {
    /**
     * Returns all of your lists (login required)
     */
    getMyLists(additionalRequestOptions?: AdditionalRequestOptions): Promise<ListQueryModel>;
    /**
     * Returns a single list given a list ID
     * @param listId ID of list
     */
    getListFromId(listId: string, additionalRequestOptions?: AdditionalRequestOptions): Promise<ListModel>;
    /**
     * Returns a list of lists that match the given name (name is case sensitive and very specific)
     * @param listName Name of list (category)
     * @param options Options for search
     * @param options.offset Offset in search (default 0)
     * @param options.limit Amount of lists to retrieve (default 10)
     */
    getListsByName(listName: string, options?: ListOptions, additionalRequestOptions?: AdditionalRequestOptions): Promise<ListQueryModel>;
    /**
     * Shortcut function to get the most popular lists
     * @param options Options for search
     */
    getPopularLists(options?: ListOptions, additionalRequestOptions?: AdditionalRequestOptions): Promise<ListQueryModel>;
    /**
     * Gets all items from a given list
     * @param listId ID of list
     * @param options Options for search
     * @param options.offset Offset in search (default 0)
     * @param options.limit Amount of lists to retrieve (default 10)
     */
    getItemsFromList(listId: string, options?: ListOptions, additionalRequestOptions?: AdditionalRequestOptions): Promise<ListItemsModel>;
    /**
     * Gets all smart lists (i.e. last purchased) of user (login required)
     */
    getMySmartLists(additionalRequestOptions?: AdditionalRequestOptions): Promise<PublicListsModel>;
    /**
     * Gets all the lists the user currently follows (login required)
     */
    getMyFollowedLists(additionalRequestOptions?: AdditionalRequestOptions): Promise<PublicListsModel>;
    /**
     * Checks whether the user is following the given list (login required)
     * @param listId ID of list
     */
    isFollowingList(listId: string, additionalRequestOptions?: AdditionalRequestOptions): Promise<boolean>;
    /**
     * Makes the user follow the given list (login required)
     * @param listId ID of list to follow
     */
    followList(listId: string, additionalRequestOptions?: AdditionalRequestOptions): Promise<any>;
    /**
     * Makes the user unfollow the given list (login required)
     * @param listId ID of list to unfollow
     */
    unfollowList(listId: string, additionalRequestOptions?: AdditionalRequestOptions): Promise<any>;
}
